@@ -62,6 +62,7 @@ static INPUT_RETURN_VALUE FcitxM17NGetCandWord(void *arg, FcitxCandidateWord *ca
 static INPUT_RETURN_VALUE FcitxM17NGetCandWords(void *arg);
 static void               FcitxM17NReload(void *arg);
 static void               FcitxM17NSave(void *arg);
+static void               FcitxM17NOnClose(void* arg, FcitxIMCloseEventType event);
 static IM*                FcitxM17NMakeIM(Addon* owner, MSymbol mlang, MSymbol mname);
 static void               FcitxM17NDelIM(IM* im);
 
@@ -342,29 +343,32 @@ MSymbol KeySymToSymbol (FcitxKeySym sym, unsigned int state)
 
     mask |= state & (FcitxKeyState_UsedMask);
 
-    const char* prefix = "";
+    // we have 7 possible below, then 20 is long enough (7 x 2 = 14 < 20)
+    char prefix[20] = "";
 
-    if (mask & FcitxKeyState_Hyper) {
-        prefix = "H-";
+    // and we use reverse order here comparing with other implementation since strcat is append.
+    // I don't know if it matters, but it's just to make sure it works.
+    if (mask & FcitxKeyState_Shift) {
+        strcat(prefix, "S-");
     }
-    if (mask & FcitxKeyState_Super) {
-        prefix = "s-";
+    if (mask & FcitxKeyState_Ctrl) {
+        strcat(prefix, "C-");
+    }
+    if (mask & FcitxKeyState_Meta) {
+        strcat(prefix, "M-");
+    }
+    if (mask & FcitxKeyState_Alt) {
+        strcat(prefix, "A-");
     }
     // This is mysterious. - xiaq
     if (mask & FcitxKeyState_ScrollLock) {
-        prefix = "G-";
+        strcat(prefix, "G-");
     }
-    if (mask & FcitxKeyState_Alt) {
-        prefix = "A-";
+    if (mask & FcitxKeyState_Super) {
+        strcat(prefix, "s-");
     }
-    if (mask & FcitxKeyState_Meta) {
-        prefix = "M-";
-    }
-    if (mask & FcitxKeyState_Ctrl) {
-        prefix = "C-";
-    }
-    if (mask & FcitxKeyState_Shift) {
-        prefix = "S-";
+    if (mask & FcitxKeyState_Hyper) {
+        strcat(prefix, "H-");
     }
 
     char* keystr;
@@ -410,7 +414,6 @@ FcitxIRV FcitxM17NDoInput(void* arg, FcitxKeySym sym, unsigned state)
 FcitxIRV FcitxM17NDoInputInternal(IM* im, FcitxKeySym sym, unsigned state)
 {
     FcitxInstance* inst = im->owner->owner;
-    FcitxInputState* is = FcitxInstanceGetInputState(inst);
     FcitxInputContext* ic = FcitxInstanceGetCurrentIC(inst);
 
     MSymbol msym = KeySymToSymbol(sym, state);
@@ -507,6 +510,7 @@ void FcitxM17NDelIM(IM* im)
 void *FcitxM17NCreate(FcitxInstance* inst)
 {
     bindtextdomain(TEXTDOMAIN, LOCALEDIR);
+    bind_textdomain_codeset(TEXTDOMAIN, "UTF-8");
 
     Addon* addon = (Addon*) fcitx_utils_malloc0(sizeof(Addon));
     addon->owner = inst;
@@ -597,6 +601,7 @@ void *FcitxM17NCreate(FcitxInstance* inst)
         iface.Save = FcitxM17NSave;
         iface.ReloadConfig = FcitxM17NReload;
         iface.GetCandWords = FcitxM17NGetCandWords;
+        iface.OnClose = FcitxM17NOnClose;
 
         int priority = 100;
         if (item && strncmp(curlang, lang, 2) == 0 && item->priority > 0)
@@ -738,3 +743,15 @@ FcitxM17NCallback (MInputContext *context,
             FcitxInstanceDeleteSurroundingText (im->owner->owner, ic, 0, -len);
     }
 }
+
+void FcitxM17NOnClose(void* arg, FcitxIMCloseEventType event)
+{
+    IM* im = (IM*) arg;
+    if (event == CET_ChangeByInactivate || event == CET_SwitchIM) {
+        if (im->owner->mic->preedit) {
+            char* preedit = MTextToUTF8(im->owner->mic->preedit);
+            FcitxInstanceCommitString(im->owner->owner, FcitxInstanceGetCurrentIC(im->owner->owner), preedit);
+            free(preedit);
+        }
+    }
+}
